@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2009 Jeffrey Stedfast
+ *  Copyright (C) 2000-2010 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -31,6 +31,7 @@
 #include "gmime-multipart.h"
 #include "gmime-part.h"
 #include "gmime-utils.h"
+#include "gmime-common.h"
 #include "gmime-stream-mem.h"
 #include "gmime-table-private.h"
 #include "gmime-parse-utils.h"
@@ -661,6 +662,7 @@ enum {
 	HEADER_SUBJECT,
 	HEADER_DATE,
 	HEADER_MESSAGE_ID,
+	HEADER_MIME_VERSION,
 	HEADER_UNKNOWN
 };
 
@@ -673,6 +675,7 @@ static const char *message_headers[] = {
 	"Subject",
 	"Date",
 	"Message-Id",
+	"MIME-Version",
 };
 
 enum {
@@ -764,6 +767,8 @@ process_header (GMimeObject *object, int action, const char *header, const char
 		g_free (message->message_id);
 		message->message_id = g_mime_utils_decode_message_id (value);
 		break;
+	case HEADER_MIME_VERSION:
+		break;
 	default:
 		return FALSE;
 	}
@@ -776,9 +781,6 @@ message_prepend_header (GMimeObject *object, const char *header, const char *val
 {
 	GMimeMessage *message = (GMimeMessage *) object;
 	
-	if (!g_ascii_strcasecmp ("MIME-Version", header))
-		return;
-	
 	/* Make sure that the header is not a Content-* header, else it
            doesn't belong on a message */
 	if (!g_ascii_strncasecmp ("Content-", header, 8))
@@ -798,9 +800,6 @@ message_append_header (GMimeObject *object, const char *header, const char *valu
 {
 	GMimeMessage *message = (GMimeMessage *) object;
 	
-	if (!g_ascii_strcasecmp ("MIME-Version", header))
-		return;
-	
 	/* Make sure that the header is not a Content-* header, else it
            doesn't belong on a message */
 	if (!g_ascii_strncasecmp ("Content-", header, 8))
@@ -820,9 +819,6 @@ message_set_header (GMimeObject *object, const char *header, const char *value)
 {
 	GMimeMessage *message = (GMimeMessage *) object;
 	
-	if (!g_ascii_strcasecmp ("MIME-Version", header))
-		return;
-	
 	/* Make sure that the header is not a Content-* header, else it
            doesn't belong on a message */
 	if (!g_ascii_strncasecmp ("Content-", header, 8))
@@ -841,18 +837,21 @@ static const char *
 message_get_header (GMimeObject *object, const char *header)
 {
 	GMimeMessage *message = (GMimeMessage *) object;
-	
-	if (!g_ascii_strcasecmp ("MIME-Version", header))
-		return "1.0";
+	const char *value;
 	
 	/* Make sure that the header is not a Content-* header, else it
            doesn't belong on a message */
-	if (g_ascii_strncasecmp ("Content-", header, 8) != 0)
-		return GMIME_OBJECT_CLASS (parent_class)->get_header (object, header);
-	else if (message->mime_part)
+	if (g_ascii_strncasecmp ("Content-", header, 8) != 0) {
+		if ((value = GMIME_OBJECT_CLASS (parent_class)->get_header (object, header)))
+			return value;
+		
+		if (!g_ascii_strcasecmp ("MIME-Version", header))
+			return "1.0";
+	} else if (message->mime_part) {
 		return g_mime_object_get_header (message->mime_part, header);
-	else
-		return NULL;
+	}
+	
+	return NULL;
 }
 
 static gboolean
@@ -863,9 +862,6 @@ message_remove_header (GMimeObject *object, const char *header)
 	GMimeRecipientType type;
 	guint i;
 	
-	if (!g_ascii_strcasecmp ("MIME-Version", header))
-		return FALSE;
-	
 	/* Make sure that the header is not a Content-* header, else it
            doesn't belong on a message */
 	if (!g_ascii_strncasecmp ("Content-", header, 8)) {
@@ -956,7 +952,8 @@ message_get_headers (GMimeObject *object)
 		
 		g_mime_header_list_write_to_stream (object->headers, stream);
 		if (message->mime_part) {
-			if (g_mime_object_get_header (message->mime_part, "Content-Type"))
+			if (g_mime_object_get_header (message->mime_part, "Content-Type") &&
+			    !g_mime_header_list_get (object->headers, "MIME-Version"))
 				g_mime_stream_write_string (stream, "MIME-Version: 1.0\n");
 			g_mime_header_list_write_to_stream (message->mime_part->headers, stream);
 		}
@@ -988,8 +985,10 @@ message_write_to_stream (GMimeObject *object, GMimeStream *stream)
 			
 			total += nwritten;
 			
-			if ((nwritten = g_mime_stream_write_string (stream, "MIME-Version: 1.0\n")) == -1)
-				return -1;
+			if (!g_mime_header_list_get (object->headers, "MIME-Version")) {
+				if ((nwritten = g_mime_stream_write_string (stream, "MIME-Version: 1.0\n")) == -1)
+					return -1;
+			}
 			
 			total += nwritten;
 		}
@@ -1072,6 +1071,9 @@ g_mime_message_set_sender (GMimeMessage *message, const char *sender)
 		g_mime_header_list_set (GMIME_OBJECT (message)->headers, "From", "");
 		message->from = NULL;
 	}
+	
+	if (message->mime_part)
+		g_mime_header_list_set_stream (message->mime_part->headers, NULL);
 }
 
 
@@ -1106,9 +1108,12 @@ g_mime_message_set_reply_to (GMimeMessage *message, const char *reply_to)
 	g_return_if_fail (reply_to != NULL);
 	
 	g_free (message->reply_to);
-	message->reply_to = g_strstrip (g_strdup (reply_to));
+	message->reply_to = g_mime_strdup_trim (reply_to);
 	
 	g_mime_header_list_set (GMIME_OBJECT (message)->headers, "Reply-To", message->reply_to);
+	
+	if (message->mime_part)
+		g_mime_header_list_set_stream (message->mime_part->headers, NULL);
 }
 
 
@@ -1146,6 +1151,9 @@ sync_recipient_header (GMimeMessage *message, GMimeRecipientType type)
 		/* list should never be NULL... */
 		g_mime_header_list_set (object->headers, name, NULL);
 	}
+	
+	if (message->mime_part)
+		g_mime_header_list_set_stream (message->mime_part->headers, NULL);
 }
 
 static void
@@ -1190,6 +1198,11 @@ g_mime_message_add_recipient (GMimeMessage *message, GMimeRecipientType type, co
 	ia = internet_address_mailbox_new (name, addr);
 	internet_address_list_add (recipients, ia);
 	g_object_unref (ia);
+	
+	if (message->mime_part)
+		g_mime_header_list_set_stream (message->mime_part->headers, NULL);
+	
+	g_mime_header_list_set_stream (((GMimeObject *) message)->headers, NULL);
 }
 
 
@@ -1262,11 +1275,14 @@ g_mime_message_set_subject (GMimeMessage *message, const char *subject)
 	g_return_if_fail (subject != NULL);
 	
 	g_free (message->subject);
-	message->subject = g_strstrip (g_strdup (subject));
+	message->subject = g_mime_strdup_trim (subject);
 	
 	encoded = g_mime_utils_header_encode_text (message->subject);
 	g_mime_object_set_header (GMIME_OBJECT (message), "Subject", encoded);
 	g_free (encoded);
+	
+	if (message->mime_part)
+		g_mime_header_list_set_stream (message->mime_part->headers, NULL);
 }
 
 
@@ -1308,6 +1324,9 @@ g_mime_message_set_date (GMimeMessage *message, time_t date, int tz_offset)
 	str = g_mime_utils_header_format_date (date, tz_offset);
 	g_mime_object_set_header (GMIME_OBJECT (message), "Date", str);
 	g_free (str);
+	
+	if (message->mime_part)
+		g_mime_header_list_set_stream (message->mime_part->headers, NULL);
 }
 
 
@@ -1374,6 +1393,9 @@ g_mime_message_set_date_as_string (GMimeMessage *message, const char *str)
 	buf = g_mime_utils_header_format_date (date, tz_offset);
 	g_mime_object_set_header (GMIME_OBJECT (message), "Date", buf);
 	g_free (buf);
+	
+	if (message->mime_part)
+		g_mime_header_list_set_stream (message->mime_part->headers, NULL);
 }
 
 
@@ -1393,11 +1415,14 @@ g_mime_message_set_message_id (GMimeMessage *message, const char *message_id)
 	g_return_if_fail (message_id != NULL);
 	
 	g_free (message->message_id);
-	message->message_id = g_strstrip (g_strdup (message_id));
+	message->message_id = g_mime_strdup_trim (message_id);
 	
 	msgid = g_strdup_printf ("<%s>", message_id);
 	g_mime_object_set_header (GMIME_OBJECT (message), "Message-Id", msgid);
 	g_free (msgid);
+	
+	if (message->mime_part)
+		g_mime_header_list_set_stream (message->mime_part->headers, NULL);
 }
 
 
