@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2009 Jeffrey Stedfast
+ *  Copyright (C) 2000-2010 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -76,8 +76,6 @@ static ssize_t write_disposition (GMimeStream *stream, const char *name, const c
 static void content_type_changed (GMimeContentType *content_type, GMimeObject *object);
 static void content_disposition_changed (GMimeContentDisposition *disposition, GMimeObject *object);
 
-static void type_registry_init (void);
-
 
 static GHashTable *type_hash = NULL;
 
@@ -127,8 +125,6 @@ g_mime_object_class_init (GMimeObjectClass *klass)
 	klass->set_content_type = set_content_type;
 	klass->get_headers = get_headers;
 	klass->write_to_stream = write_to_stream;
-	
-	type_registry_init ();
 }
 
 static void
@@ -280,8 +276,6 @@ g_mime_object_register_type (const char *type, const char *subtype, GType object
 	g_return_if_fail (subtype != NULL);
 	g_return_if_fail (type != NULL);
 	
-	type_registry_init ();
-	
 	if (!(bucket = g_hash_table_lookup (type_hash, type))) {
 		bucket = g_new (struct _type_bucket, 1);
 		bucket->type = g_strdup (type);
@@ -322,8 +316,6 @@ g_mime_object_new (GMimeContentType *content_type)
 	
 	g_return_val_if_fail (GMIME_IS_CONTENT_TYPE (content_type), NULL);
 	
-	type_registry_init ();
-	
 	if ((bucket = g_hash_table_lookup (type_hash, content_type->type))) {
 		if (!(sub = g_hash_table_lookup (bucket->subtype_hash, content_type->subtype)))
 			sub = g_hash_table_lookup (bucket->subtype_hash, "*");
@@ -377,8 +369,6 @@ g_mime_object_new_type (const char *type, const char *subtype)
 	
 	g_return_val_if_fail (type != NULL, NULL);
 	
-	type_registry_init ();
-	
 	if ((bucket = g_hash_table_lookup (type_hash, type))) {
 		if (!(sub = g_hash_table_lookup (bucket->subtype_hash, subtype)))
 			sub = g_hash_table_lookup (bucket->subtype_hash, "*");
@@ -786,7 +776,7 @@ prepend_header (GMimeObject *object, const char *header, const char *value)
  * @header: header name
  * @value: header value
  *
- * Prepends a header to the MIME object.
+ * Prepends a raw, unprocessed header to the MIME object.
  **/
 void
 g_mime_object_prepend_header (GMimeObject *object, const char *header, const char *value)
@@ -812,7 +802,7 @@ append_header (GMimeObject *object, const char *header, const char *value)
  * @header: header name
  * @value: header value
  *
- * Appends a header to the MIME object.
+ * Appends a raw, unprocessed header to the MIME object.
  **/
 void
 g_mime_object_append_header (GMimeObject *object, const char *header, const char *value)
@@ -839,7 +829,7 @@ set_header (GMimeObject *object, const char *header, const char *value)
  * @header: header name
  * @value: header value
  *
- * Sets an arbitrary header on the MIME object.
+ * Sets an arbitrary raw, unprocessed header on the MIME object.
  **/
 void
 g_mime_object_set_header (GMimeObject *object, const char *header, const char *value)
@@ -864,11 +854,10 @@ get_header (GMimeObject *object, const char *header)
  * @object: a #GMimeObject
  * @header: header name
  *
- * Gets the value of the requested header if it exists or %NULL
- * otherwise.
+ * Gets the raw, unprocessed value of the requested header.
  *
- * Returns: the value of the header @header if it exists or %NULL
- * otherwise.
+ * Returns: the raw, unprocessed value of the requested header if it
+ * exists or %NULL otherwise.
  **/
 const char *
 g_mime_object_get_header (GMimeObject *object, const char *header)
@@ -1061,20 +1050,19 @@ type_bucket_foreach (gpointer key, gpointer value, gpointer user_data)
 	g_free (bucket);
 }
 
-static void
-type_registry_shutdown (void)
+void
+g_mime_object_type_registry_shutdown (void)
 {
 	g_hash_table_foreach (type_hash, type_bucket_foreach, NULL);
 	g_hash_table_destroy (type_hash);
+	type_hash = NULL;
 }
 
-static void
-type_registry_init (void)
+void
+g_mime_object_type_registry_init (void)
 {
 	if (type_hash)
 		return;
 	
 	type_hash = g_hash_table_new (g_mime_strcase_hash, g_mime_strcase_equal);
-	
-	g_atexit (type_registry_shutdown);
 }
