@@ -1182,7 +1182,7 @@ header_fold (const char *in, gboolean structured)
 		len = strcspn (inptr, " \t\n");
 		
 		if (len > 1 && outlen + len > GMIME_FOLD_LEN) {
-			if (outlen > 1 && out->len > fieldlen + 2) {
+			if (outlen > 1 && out->len >= fieldlen + 2) {
 				if (last_was_lwsp) {
 					if (structured)
 						out->str[out->len - 1] = '\t';
@@ -1190,6 +1190,7 @@ header_fold (const char *in, gboolean structured)
 					g_string_insert_c (out, out->len - 1, '\n');
 				} else
 					g_string_append (out, "\n\t");
+				
 				outlen = 1;
 			}
 			
@@ -1747,7 +1748,7 @@ quoted_decode (const unsigned char *in, size_t len, unsigned char *out, int *sta
 				
 				goto decode;
 			}
-
+			
 			saved = 0;
 			need = 0;
 			
@@ -1906,6 +1907,10 @@ rfc2047_token_new_encoded_word (const char *word, size_t len)
 	/* find the end of the payload */
 	inptr = word + len - 2;
 	
+	/* make sure that we don't have something like: =?iso-8859-1?Q?= */
+	if (payload > inptr)
+		return NULL;
+	
 	token = rfc2047_token_new (payload, inptr - payload);
 	token->charset = g_mime_charset_iconv_name (charset);
 	token->encoding = encoding;
@@ -1939,6 +1944,7 @@ tokenize_rfc2047_phrase (const char *in, size_t *len)
 			lwsp = NULL;
 		
 		word = inptr;
+		ascii = TRUE;
 		if (is_atom (*inptr)) {
 			if (G_UNLIKELY (enable_rfc2047_workarounds)) {
 				/* Make an extra effort to detect and
@@ -2266,6 +2272,7 @@ rfc2047_decode_tokens (rfc2047_token *tokens, size_t buflen)
 				g_mime_iconv_close (cd);
 				
 				g_string_append_len (decoded, str, len);
+				g_free (str);
 				
 #if w(!)0
 				if (ninval > 0) {
@@ -2497,7 +2504,7 @@ rfc2047_encode_get_rfc822_words (const char *in, gboolean phrase)
 		
 		inptr = newinptr;
 		
-		if (c < 256 && is_lwsp (c)) {
+		if (c < 256 && is_blank (c)) {
 			if (count > 0) {
 				word = rfc822_word_new ();
 				word->next = NULL;
@@ -2516,19 +2523,26 @@ rfc2047_encode_get_rfc822_words (const char *in, gboolean phrase)
 			encoding = 0;
 		} else {
 			count++;
-			if (phrase && c < 128) {
-				/* phrases can have qstring words */
-				if (!is_atom (c))
+			if (c < 128) {
+				if (is_ctrl (c)) {
+					type = WORD_2047;
+					encoding = MAX (encoding, 1);
+				} else if (phrase && !is_atom (c)) {
+					/* phrases can have qstring words */
 					type = MAX (type, WORD_QSTRING);
-			} else if (c > 127 && c < 256) {
+				}
+			} else if (c < 256) {
 				type = WORD_2047;
 				encoding = MAX (encoding, 1);
-			} else if (c >= 256) {
+			} else {
 				type = WORD_2047;
 				encoding = 2;
 			}
 			
 			if (count >= GMIME_FOLD_PREENCODED) {
+				if (type == WORD_ATOM)
+					type = WORD_2047;
+				
 				word = rfc822_word_new ();
 				word->next = NULL;
 				word->start = start;
