@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2009 Jeffrey Stedfast
+ *  Copyright (C) 2000-2010 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -245,7 +245,7 @@ int main (int argc, char **argv)
 	DIR *dir;
 	
 #ifdef ENABLE_MBOX_MATCH
-	if (g_mkdir ("./tmp", 0755) == -1 && errno != EEXIST)
+	if (mkdir ("./tmp", 0755) == -1 && errno != EEXIST)
 		return 0;
 #endif
 	
@@ -437,7 +437,19 @@ int main (int argc, char **argv)
  exit:
 	
 #ifdef ENABLE_MBOX_MATCH
-	g_rmdir ("./tmp");
+	if ((dir = g_dir_open ("./tmp", 0, NULL))) {
+		p = g_stpcpy (input, "./tmp");
+		*p++ = G_DIR_SEPARATOR;
+		
+		while ((dent = g_dir_read_name (dir))) {
+			strcpy (p, dent);
+			unlink (input);
+		}
+		
+		g_dir_close (dir);
+	}
+	
+	rmdir ("./tmp");
 #endif
 	
 	testsuite_end ();
